package sun.awt.util;

import java.util.ConcurrentModificationException;
import java.lang.reflect.Array;
import java.util.Iterator;
import java.util.ListIterator;
import java.util.NoSuchElementException;
import java.util.Collection;
import java.util.Deque;
import java.util.List;
import java.util.AbstractSequentialList;

public class IdentityLinkedList<E> extends AbstractSequentialList<E> implements List<E>, Deque<E>
{
    private transient IdentityLinkedList.Entry<E> header;
    private transient int size;
    
    public IdentityLinkedList() {
        this.header = new IdentityLinkedList.Entry(null);
        this.size = 0;
        IdentityLinkedList.Entry<E> header = this.header;
        IdentityLinkedList.Entry<E> header2 = this.header;
        IdentityLinkedList.Entry<E> header3 = this.header;
        header2.previous = header3;
        header.next = header3;
    }
    
    public IdentityLinkedList(Collection<? extends E> collection) {
        this();
        this.addAll(collection);
    }
    
    @Override
    public E getFirst() {
        if (this.size == 0) {
            throw new NoSuchElementException();
        }
        return this.header.next.element;
    }
    
    @Override
    public E getLast() {
        if (this.size == 0) {
            throw new NoSuchElementException();
        }
        return this.header.previous.element;
    }
    
    @Override
    public E removeFirst() {
        return this.remove(this.header.next);
    }
    
    @Override
    public E removeLast() {
        return this.remove(this.header.previous);
    }
    
    @Override
    public void addFirst(E e) {
        this.addBefore(e, this.header.next);
    }
    
    @Override
    public void addLast(E e) {
        this.addBefore(e, this.header);
    }
    
    @Override
    public boolean contains(Object o) {
        return this.indexOf(o) != -1;
    }
    
    @Override
    public int size() {
        return this.size;
    }
    
    @Override
    public boolean add(E e) {
        this.addBefore(e, this.header);
        return true;
    }
    
    @Override
    public boolean remove(Object o) {
        IdentityLinkedList.Entry<E> entry = this.header.next;
        while (entry != this.header) {
            if (o == entry.element) {
                this.remove(entry);
                return true;
            }
            entry = entry.next;
        }
        return false;
    }
    
    @Override
    public boolean addAll(Collection<? extends E> collection) {
        return this.addAll(this.size, collection);
    }
    
    @Override
    public boolean addAll(int i, Collection<? extends E> collection) {
        if (i < 0 || i > this.size) {
            throw new IndexOutOfBoundsException("Index: " + i + ", Size: " + this.size);
        }
        Object[] array = collection.toArray();
        int length = array.length;
        if (length == 0) {
            return false;
        }
        ++this.modCount;
        IdentityLinkedList.Entry<Object> entry = (i == this.size) ? this.header : this.entry(i);
        IdentityLinkedList.Entry<Object> previous = entry.previous;
        for (int j = 0; j < length; ++j) {
            IdentityLinkedList.Entry next = new IdentityLinkedList.Entry<Object>(array[j], entry, previous);
            previous.next = next;
            previous = next;
        }
        entry.previous = previous;
        this.size += length;
        return true;
    }
    
    @Override
    public void clear() {
        IdentityLinkedList.Entry<Object> next2;
        for (IdentityLinkedList.Entry next = this.header.next; next != this.header; next = next2) {
            next2 = next.next;
            IdentityLinkedList.Entry<E> entry = next;
            IdentityLinkedList.Entry<E> entry2 = null;
            next.previous = entry2;
            entry.next = entry2;
            next.element = null;
        }
        IdentityLinkedList.Entry<E> header = this.header;
        IdentityLinkedList.Entry<E> header2 = this.header;
        IdentityLinkedList.Entry<E> header3 = this.header;
        header2.previous = header3;
        header.next = header3;
        this.size = 0;
        ++this.modCount;
    }
    
    @Override
    public E get(int n) {
        return this.entry(n).element;
    }
    
    @Override
    public E set(int n, E element) {
        IdentityLinkedList.Entry<E> entry = this.entry(n);
        Object element2 = entry.element;
        entry.element = element;
        return element2;
    }
    
    @Override
    public void add(int n, E e) {
        this.addBefore(e, (n == this.size) ? this.header : this.entry(n));
    }
    
    @Override
    public E remove(int n) {
        return this.remove(this.entry(n));
    }
    
    private IdentityLinkedList.Entry<E> entry(int i) {
        if (i < 0 || i >= this.size) {
            throw new IndexOutOfBoundsException("Index: " + i + ", Size: " + this.size);
        }
        IdentityLinkedList.Entry<Object> entry = this.header;
        if (i < this.size >> 1) {
            for (int k = 0; k <= i; ++k) {
                entry = entry.next;
            }
            return entry;
        }
        for (int j = this.size; j > i; --j) {
            entry = entry.previous;
        }
        return entry;
    }
    
    @Override
    public int indexOf(Object o) {
        int n = 0;
        IdentityLinkedList.Entry<Object> entry = this.header.next;
        while (entry != this.header) {
            if (o == entry.element) {
                return n;
            }
            ++n;
            entry = entry.next;
        }
        return -1;
    }
    
    @Override
    public int lastIndexOf(Object o) {
        int size = this.size;
        IdentityLinkedList.Entry<Object> entry = this.header.previous;
        while (entry != this.header) {
            --size;
            if (o == entry.element) {
                return size;
            }
            entry = entry.previous;
        }
        return -1;
    }
    
    @Override
    public E peek() {
        if (this.size == 0) {
            return null;
        }
        return this.getFirst();
    }
    
    @Override
    public E element() {
        return this.getFirst();
    }
    
    @Override
    public E poll() {
        if (this.size == 0) {
            return null;
        }
        return this.removeFirst();
    }
    
    @Override
    public E remove() {
        return this.removeFirst();
    }
    
    @Override
    public boolean offer(E e) {
        return this.add(e);
    }
    
    @Override
    public boolean offerFirst(E e) {
        this.addFirst(e);
        return true;
    }
    
    @Override
    public boolean offerLast(E e) {
        this.addLast(e);
        return true;
    }
    
    @Override
    public E peekFirst() {
        if (this.size == 0) {
            return null;
        }
        return this.getFirst();
    }
    
    @Override
    public E peekLast() {
        if (this.size == 0) {
            return null;
        }
        return this.getLast();
    }
    
    @Override
    public E pollFirst() {
        if (this.size == 0) {
            return null;
        }
        return this.removeFirst();
    }
    
    @Override
    public E pollLast() {
        if (this.size == 0) {
            return null;
        }
        return this.removeLast();
    }
    
    @Override
    public void push(E e) {
        this.addFirst(e);
    }
    
    @Override
    public E pop() {
        return this.removeFirst();
    }
    
    @Override
    public boolean removeFirstOccurrence(Object o) {
        return this.remove(o);
    }
    
    @Override
    public boolean removeLastOccurrence(Object o) {
        IdentityLinkedList.Entry<E> entry = this.header.previous;
        while (entry != this.header) {
            if (o == entry.element) {
                this.remove(entry);
                return true;
            }
            entry = entry.previous;
        }
        return false;
    }
    
    @Override
    public ListIterator<E> listIterator(int n) {
        return new IdentityLinkedList.ListItr(n);
    }
    
    private IdentityLinkedList.Entry<E> addBefore(E e, IdentityLinkedList.Entry<E> entry) {
        IdentityLinkedList.Entry entry2 = new IdentityLinkedList.Entry(e, entry, entry.previous);
        entry2.previous.next = entry2;
        entry2.next.previous = entry2;
        ++this.size;
        ++this.modCount;
        return entry2;
    }
    
    private E remove(IdentityLinkedList.Entry<E> entry) {
        if (entry == this.header) {
            throw new NoSuchElementException();
        }
        Object element = entry.element;
        entry.previous.next = entry.next;
        entry.next.previous = entry.previous;
        IdentityLinkedList.Entry<E> entry2 = null;
        entry.previous = entry2;
        entry.next = entry2;
        entry.element = null;
        --this.size;
        ++this.modCount;
        return element;
    }
    
    @Override
    public Iterator<E> descendingIterator() {
        return new IdentityLinkedList.DescendingIterator();
    }
    
    @Override
    public Object[] toArray() {
        Object[] array = new Object[this.size];
        int n = 0;
        for (IdentityLinkedList.Entry entry = this.header.next; entry != this.header; entry = entry.next) {
            array[n++] = entry.element;
        }
        return array;
    }
    
    @Override
    public <T> T[] toArray(T[] array) {
        if (array.length < this.size) {
            array = (Object[])(Object[])Array.newInstance(array.getClass().getComponentType(), this.size);
        }
        int n = 0;
        for (IdentityLinkedList.Entry entry = this.header.next; entry != this.header; entry = entry.next) {
            array[n++] = entry.element;
        }
        if (array.length > this.size) {
            array[this.size] = null;
            return array;
        }
        return array;
    }
    
    static Object access$300(IdentityLinkedList list, IdentityLinkedList.Entry entry) {
        return list.remove(entry);
    }
    
    static IdentityLinkedList.Entry access$400(IdentityLinkedList list, Object o, IdentityLinkedList.Entry entry) {
        return list.addBefore(o, entry);
    }
    
    private class ListItr implements ListIterator<E>
    {
        private IdentityLinkedList.Entry<E> lastReturned;
        private IdentityLinkedList.Entry<E> next;
        private int nextIndex;
        private int expectedModCount;
        
        ListItr(int i) {
            this.lastReturned = IdentityLinkedList.this.header;
            this.expectedModCount = IdentityLinkedList.this.modCount;
            if (i < 0 || i > IdentityLinkedList.this.size) {
                throw new IndexOutOfBoundsException("Index: " + i + ", Size: " + IdentityLinkedList.this.size);
            }
            if (i < IdentityLinkedList.this.size >> 1) {
                this.next = IdentityLinkedList.this.header.next;
                this.nextIndex = 0;
                while (this.nextIndex < i) {
                    this.next = this.next.next;
                    ++this.nextIndex;
                }
            }
            else {
                this.next = IdentityLinkedList.this.header;
                this.nextIndex = IdentityLinkedList.this.size;
                while (this.nextIndex > i) {
                    this.next = this.next.previous;
                    --this.nextIndex;
                }
            }
            return;
        }
        
        @Override
        public boolean hasNext() {
            return this.nextIndex != IdentityLinkedList.this.size;
        }
        
        @Override
        public E next() {
            this.checkForComodification();
            if (this.nextIndex == IdentityLinkedList.this.size) {
                throw new NoSuchElementException();
            }
            this.lastReturned = this.next;
            this.next = this.next.next;
            ++this.nextIndex;
            return this.lastReturned.element;
        }
        
        @Override
        public boolean hasPrevious() {
            return this.nextIndex != 0;
        }
        
        @Override
        public E previous() {
            if (this.nextIndex == 0) {
                throw new NoSuchElementException();
            }
            IdentityLinkedList.Entry<E> previous = this.next.previous;
            this.next = previous;
            this.lastReturned = previous;
            --this.nextIndex;
            this.checkForComodification();
            return this.lastReturned.element;
        }
        
        @Override
        public int nextIndex() {
            return this.nextIndex;
        }
        
        @Override
        public int previousIndex() {
            return this.nextIndex - 1;
        }
        
        @Override
        public void remove() {
            this.checkForComodification();
            IdentityLinkedList.Entry<Object> next = this.lastReturned.next;
            try {
                IdentityLinkedList.access$300(IdentityLinkedList.this, this.lastReturned);
            }
            catch (NoSuchElementException ex) {
                throw new IllegalStateException();
            }
            if (this.next == this.lastReturned) {
                this.next = next;
            }
            else {
                --this.nextIndex;
            }
            this.lastReturned = IdentityLinkedList.this.header;
            ++this.expectedModCount;
        }
        
        @Override
        public void set(E element) {
            if (this.lastReturned == IdentityLinkedList.this.header) {
                throw new IllegalStateException();
            }
            this.checkForComodification();
            this.lastReturned.element = element;
            return;
        }
        
        @Override
        public void add(E e) {
            this.checkForComodification();
            this.lastReturned = IdentityLinkedList.this.header;
            IdentityLinkedList.access$400(IdentityLinkedList.this, e, this.next);
            ++this.nextIndex;
            ++this.expectedModCount;
        }
        
        final void checkForComodification() {
            if (IdentityLinkedList.this.modCount != this.expectedModCount) {
                throw new ConcurrentModificationException();
            }
            return;
        }
    }
    
    private static class Entry<E>
    {
        E element;
        IdentityLinkedList.Entry<E> next;
        IdentityLinkedList.Entry<E> previous;
        
        Entry(E element, IdentityLinkedList.Entry<E> next, IdentityLinkedList.Entry<E> previous) {
            this.element = element;
            this.next = next;
            this.previous = previous;
        }
    }
    
    private class DescendingIterator implements Iterator
    {
        final IdentityLinkedList.ListItr itr;
        
        private DescendingIterator() {
            this.itr = new IdentityLinkedList.ListItr(IdentityLinkedList.this.size());
        }
        
        @Override
        public boolean hasNext() {
            return this.itr.hasPrevious();
        }
        
        @Override
        public E next() {
            return this.itr.previous();
        }
        
        @Override
        public void remove() {
            this.itr.remove();
        }
    }
}
